////////////////////////////////////////////////////////////////////////////////
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /    Vendor: Xilinx 
// \   \   \/     Version : 2.1
//  \   \         Application : 7 Series FPGAs Transceivers Wizard 
//  /   /         Filename : jesd204b_rx1_gtwizard_v2_1_top.v
// /___/   /\     
// \   \  /  \ 
//  \___\/\___\ 
//
//
// Module jesd204b_rx1_gtwizard_v2_1_top
// Generated by Xilinx 7 Series FPGAs Transceivers Wizard
// 
// 
// (c) Copyright 2010-2011 Xilinx, Inc. All rights reserved.
// 
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
// 
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
// 
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
// 
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES. 


`timescale 1ns / 1ps

//***********************************Entity Declaration************************

module jesd204b_rx1_gtwizard_v2_1_top #
(
    parameter EXAMPLE_SIM_GTRESET_SPEEDUP          =   "TRUE"    // simulation setting for GT SecureIP model

)
(
    // Reset Inputs for each direction
    input               txreset,
    input               rxreset,
    
    // Refclk input pads
    input               refclk,

    // Serial ports
    output     [0:0]    txp_out,
    output     [0:0]    txn_out,
    input      [0:0]    rxp_in,
    input      [0:0]    rxn_in,
    
    // Reset Done for each direction
    output              tx_reset_done, 
    output              rx_reset_done, 

    // Clocks
    // txsusrclk is sourced externally to allow use of a dedicated clock for JESD204B Subclass 1
    // In other modes txusrclk can be sourced from txoutclk
    output              txoutclk,
    output              rxoutclk,
    input               txusrclk, 
    input               txusrclkrdy, 
    input               rxusrclk, 
    input               rxusrclkrdy, 

    // Tx Ports
    input     [31:0]    txdata,
    input      [3:0]    txcharisk,
    
    //Rx Ports
    output    [31:0]    rxdata,
    output     [3:0]    rxcharisk,
    output     [3:0]    rxdisperr,
    output     [3:0]    rxnotintable,
    input               rxencommaalign,

    // Channel Bonding (A only)
    input               rxenchanbond,
    output              rxchanbondseq,
    
    // DRP
    input               drpclk,
    input      [8:0]    gt0_drpaddr, 
    input      [15:0]   gt0_drpdi,   
    input               gt0_drpen,   
    input               gt0_drpwe,
    output     [15:0]   gt0_drpdo,
    output              gt0_drprdy
    
);



//************************** Register Declarations ****************************

    reg             gt0_txresetdone_r;
    reg             gt0_rxresetdone_r;

    wire            gt0_cplllock_i;
    wire            gt0_cpllreset_i;

    wire            gt0_rxresetdone_i;
    wire            gt0_txresetdone_i;

    reg             txresetdone_r2;
    reg             rxresetdone_r2;
    reg             txresetdone_r3;
    reg             rxresetdone_r3;
    reg [15:0]      tx_reset_counter = 16'hFF;
    reg [15:0]      rx_reset_counter = 16'hFF;
    wire            qplllock_i;
    wire            qplllock0_i;
    wire            qplllock1_i;
    wire            qpllreset_i;




    //----------------------------- Global Signals -----------------------------

    wire            txreset_i;
    wire            rxreset_i;
 
    //--------------------------- Reference Clocks ----------------------------
    

    //------------------------------- Data Bus -------------------------------

    wire      [31:0]   gt0_txdata_i;
    wire      [3:0]    gt0_txcharisk_i;
    
    // Rx Ports
    wire      [31:0]   gt0_rxdata_i;
    wire      [3:0]    gt0_rxcharisk_i;
    wire      [3:0]    gt0_rxdisperr_i;
    wire      [3:0]    gt0_rxnotintable_i;
        

//**************************** Main Body of Code *******************************

    // See the 7 Series GTX User Guide for Ref Clock details
    // If Tx = CPLL and Rx = QPLL use two reference clocks. If the PLL is shared
    // then only one refclk is required. Comment out the unused IBUFDS to avoid
    // synthesis errors if the IBUFDS is not driven.

    assign gt0_txcharisk_i = txcharisk[3:0];
    assign gt0_txdata_i = txdata[31:0]; 

    assign rxdata =       gt0_rxdata_i;
    assign rxcharisk =    gt0_rxcharisk_i;
    assign rxdisperr =    gt0_rxdisperr_i; 
    assign rxnotintable = gt0_rxnotintable_i; 

    //***********************************************************************//
    //                                                                       //
    //--------------------------- The GT Wrapper ----------------------------//
    //                                                                       //
    //***********************************************************************//
        
    jesd204b_rx1_gtwizard_v2_1 #
    (
        .WRAPPER_SIM_GTRESET_SPEEDUP   (EXAMPLE_SIM_GTRESET_SPEEDUP)
    )
    gtwizard_v2_1_i
    (
 
 

        //_____________________________________________________________________
        //_____________________________________________________________________
        //GT0  (X0Y0)

        //-------------- Channel - Dynamic Reconfiguration Port (DRP) --------------
        .GT0_DRPADDR_IN                 (gt0_drpaddr),
        .GT0_DRPCLK_IN                  (drpclk),
        .GT0_DRPDI_IN                   (gt0_drpdi),
        .GT0_DRPDO_OUT                  (gt0_drpdo),
        .GT0_DRPEN_IN                   (gt0_drpen),
        .GT0_DRPRDY_OUT                 (gt0_drprdy),
        .GT0_DRPWE_IN                   (gt0_drpwe),
        //----------------------- Channel - Ref Clock Ports ------------------------
        .GT0_GTREFCLK0_IN               (refclk),
        //------------------------------ Channel PLL -------------------------------
        .GT0_CPLLFBCLKLOST_OUT          (),
        .GT0_CPLLLOCK_OUT               (gt0_cplllock_i),
        .GT0_CPLLLOCKDETCLK_IN          (drpclk),
        .GT0_CPLLREFCLKLOST_OUT         (),
        .GT0_CPLLRESET_IN               (gt0_cpllreset_i),
        //----------------------------- Eye Scan Ports -----------------------------
        .GT0_EYESCANDATAERROR_OUT       (),
        //---------------------- Loopback and Powerdown Ports ----------------------
        .GT0_LOOPBACK_IN                (3'd0),
        //----------------------------- Receive Ports ------------------------------
        .GT0_RXUSERRDY_IN               (rxusrclkrdy),
        //--------------------- Receive Ports - 8b10b Decoder ----------------------
        .GT0_RXCHARISCOMMA_OUT          (),
        .GT0_RXCHARISK_OUT              (gt0_rxcharisk_i),
        .GT0_RXDISPERR_OUT              (gt0_rxdisperr_i),
        .GT0_RXNOTINTABLE_OUT           (gt0_rxnotintable_i),
        //----------------- Receive Ports - Channel Bonding Ports ------------------
        .GT0_RXCHANBONDSEQ_OUT          (),
        .GT0_RXCHBONDEN_IN              (rxenchanbond),
        .GT0_RXCHBONDI_IN               (5'b00000),
        .GT0_RXCHBONDLEVEL_IN           (3'b000), 
        .GT0_RXCHBONDMASTER_IN          (1'b1),           
        .GT0_RXCHBONDO_OUT              (),   
        .GT0_RXCHBONDSLAVE_IN           (1'b0),           
        //----------------- Receive Ports - Channel Bonding Ports  -----------------
        .GT0_RXCHANISALIGNED_OUT        (),          
        .GT0_RXCHANREALIGN_OUT          (),          
        .GT0_RXMCOMMAALIGNEN_IN         (rxencommaalign),
        .GT0_RXPCOMMAALIGNEN_IN         (rxencommaalign),
        //----------------- Receive Ports - RX Data Path interface -----------------
        .GT0_GTRXRESET_IN               (rxreset_i),
        .GT0_RXDATA_OUT                 (gt0_rxdata_i),
        .GT0_RXOUTCLK_OUT               (rxoutclk),
        .GT0_RXPCSRESET_IN              (1'b0),
        .GT0_RXUSRCLK_IN                (rxusrclk),
        .GT0_RXUSRCLK2_IN               (rxusrclk),
        //----- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
        .GT0_GTXRXN_IN                  (rxn_in[0]),
        .GT0_GTXRXP_IN                  (rxp_in[0]),
        .GT0_RXCDRLOCK_OUT              (),
        .GT0_RXELECIDLE_OUT             (),
        //---------------------- Receive Ports - RX PLL Ports ----------------------
        .GT0_RXRESETDONE_OUT            (gt0_rxresetdone_i),
        //----------------------------- Transmit Ports -----------------------------
        .GT0_TXPOSTCURSOR_IN            (5'd0),
        .GT0_TXPRECURSOR_IN             (5'd0),
        .GT0_TXDIFFCTRL_IN              (4'b1000),
        .GT0_TXPOLARITY_IN              (1'b0),
        .GT0_TXUSERRDY_IN               (txusrclkrdy),
        //-------------- Transmit Ports - 8b10b Encoder Control Ports --------------
        .GT0_TXCHARISK_IN               (gt0_txcharisk_i),
        //---------------- Transmit Ports - TX Data Path interface -----------------
        .GT0_GTTXRESET_IN               (txreset_i),
        .GT0_TXDATA_IN                  (gt0_txdata_i),
        .GT0_TXOUTCLK_OUT               (txoutclk),
        .GT0_TXOUTCLKFABRIC_OUT         (),
        .GT0_TXOUTCLKPCS_OUT            (),
        .GT0_TXPCSRESET_IN              (1'b0),
        .GT0_TXUSRCLK_IN                (txusrclk),
        .GT0_TXUSRCLK2_IN               (txusrclk),
        //-------------- Transmit Ports - TX Driver and OOB signaling --------------
        .GT0_GTXTXN_OUT                 (txn_out[0]),
        .GT0_GTXTXP_OUT                 (txp_out[0]),
        //--------------------- Transmit Ports - TX PLL Ports ----------------------
        .GT0_TXRESETDONE_OUT            (gt0_txresetdone_i)
 


    );

    assign qplllock_i = 1'b1;

    // See the 7 Series GTX User Guide for PLL details
    // If Tx = CPLL and Rx = QPLL set TXSYSCLKSEL = 2'b00 and RXSYSCLKSEL = 2'b11 and uncomment the following lines of code:
    // assign  gt0_cpllreset_i = txreset;
    // assign  qpllreset_i = rxreset;
    // assign  rxreset_i = rxreset | !qplllock_i;
    // assign  txreset_i = txreset | !gt0_cplllock_i;
    
    // If Rx = CPLL and Tx = QPLL set TXSYSCLKSEL = 2'b11 and RXSYSCLKSEL = 2'b00 and uncomment the following lines of code:
    //   assign  gt0_cpllreset_i = rxreset;
    //   assign  qpllreset_i = txreset;
    //   assign  rxreset_i = rxreset | !gt0_cplllock_i;
    //   assign  txreset_i = txreset | !qplllock_i;
    
    // If Rx and Tx = CPLL set TXSYSCLKSEL = 2'b00 and RXSYSCLKSEL = 2'b00 and uncomment the following lines of code:
    assign  gt0_cpllreset_i = rxreset | txreset;
    assign  qpllreset_i = 1'b0;
    assign  rxreset_i = rxreset | !gt0_cplllock_i;
    assign  txreset_i = txreset | !gt0_cplllock_i;
    
    // If Rx and Tx = QPLL set TXSYSCLKSEL = 2'b11 and RXSYSCLKSEL = 2'b11 and uncomment the following lines of code:
    //   assign  qpllreset_i = rxreset | txreset;
    //   assign  gt0_cpllreset_i = 1'b0;
    //   assign  rxreset_i = rxreset | !qplllock_i;
    //   assign  txreset_i = txreset | !qplllock_i;
 
    always @(posedge drpclk)
    begin
      if (txreset_i)
        tx_reset_counter <= 16'hFF;
      else
        tx_reset_counter <= {1'b0, tx_reset_counter[15:1]};
    end

    always @(posedge drpclk)
    begin
      if (rxreset_i)
        rx_reset_counter <= 16'hFF;
      else
        rx_reset_counter <= {1'b0, rx_reset_counter[15:1]};
    end
    
    always @(posedge rxusrclk or negedge gt0_rxresetdone_i)
      if (!gt0_rxresetdone_i)
        gt0_rxresetdone_r    <=   1'b0;
      else
        gt0_rxresetdone_r    <=   gt0_rxresetdone_i;
    
    always @(posedge txusrclk or negedge gt0_txresetdone_i)
      if (!gt0_txresetdone_i)
        gt0_txresetdone_r    <=   1'b0;
      else
        gt0_txresetdone_r    <=   gt0_txresetdone_i; 
   
    always @(posedge txusrclk)
    begin
      rxresetdone_r2   <=   gt0_rxresetdone_r;
      rxresetdone_r3   <=   rxresetdone_r2;
      txresetdone_r2   <=   gt0_txresetdone_r;
      txresetdone_r3   <=   txresetdone_r2;
    end
 
    assign tx_reset_done = txresetdone_r3; 
    assign rx_reset_done = rxresetdone_r3; 

  endmodule
