// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************

`timescale 1ns/100ps

module jesd204b_rx1_top (

  // Physical interface, except for data, all other inputs/outputs must be buffered
  // outside this module. Refer to the utility cores for the jesd buffers.

  ref_clk_in,
  rx_data_p,
  rx_data_n,
  rx_sync,
  rx_sysref,

  // these are the outputs of the core

  rx_clk,
  rx_data0,

  // master interface and status signals

  mb_req,
  mb_ack,
  mb_addr,
  mb_data,
  mb_done,
  mb_error,
  mb_readyn,

  // processor interface (see regmap.txt at the pcore directory for details)

  up_rstn,
  up_clk,
  up_sel,
  up_rwn,
  up_addr,
  up_wdata,
  up_rdata,
  up_ack,

  // debug interface (chipscope)

  es_dbg_data,
  es_dbg_trigger,

  // es debug data (for chipscope)

  mb_dbg_data,
  mb_dbg_trigger,

  // debug interface (chipscope)

  debug_data,
  debug_trigger);

  parameter C_CF_BUFTYPE = 0;

  // Physical interface, except for data, all other inputs/outputs must be buffered
  // outside this module. Refer to the utility cores for the jesd buffers.

  input           ref_clk_in;
  input           rx_data_p;
  input           rx_data_n;
  output          rx_sync;
  output          rx_sysref;

  // these are the outputs of the jesd core, this interface is same as the xilinx ip

  output          rx_clk;
  output  [31:0]  rx_data0;

  // master interface and status signals

  output          mb_req;
  input           mb_ack;
  output  [31: 0] mb_addr;
  output  [31: 0] mb_data;
  input           mb_done;
  input           mb_error;
  input           mb_readyn;

  // processor interface (see regmap.txt at the pcore directory for details)

  input           up_rstn;
  input           up_clk;
  input           up_sel;
  input           up_rwn;
  input   [ 4:0]  up_addr;
  input   [31:0]  up_wdata;
  output  [31:0]  up_rdata;
  output          up_ack;

  // debug interface (chipscope)

  output [227:0]  es_dbg_data;
  output  [ 7:0]  es_dbg_trigger;

  // es debug data (for chipscope)

  output  [83:0]  mb_dbg_data;
  output  [ 7:0]  mb_dbg_trigger;

  // debug interface (chipscope)

  output [107:0]  debug_data;
  output  [ 5:0]  debug_trigger;

  // internal registers

  reg             rx_sysref_enb = 'd0;
  reg             rx_sysref_enb_d = 'd0;
  reg             rx_sysref = 'd0;
  reg             rx_sync_enb = 'd0;
  reg             rx_sync = 'd0;
  reg             rx_status = 'd0;
 
  // internal signals

  wire            up_lanesync_enb_s;
  wire            up_scr_enb_s;
  wire            up_sysref_enb_s;
  wire            up_err_disb_s;
  wire    [ 4:0]  up_frmcnt_s;
  wire    [ 7:0]  up_bytecnt_s;
  wire    [12:0]  up_bufdelay_s;
  wire    [ 1:0]  up_test_mode_s;
  wire            up_gtx_rstn_s;
  wire            up_jesd_rstn_s;
  wire            up_es_mode_s;
  wire            up_es_start_s;
  wire    [ 4:0]  up_prescale_s;
  wire    [ 7:0]  up_voffset_step_s;
  wire    [ 7:0]  up_voffset_max_s;
  wire    [ 7:0]  up_voffset_min_s;
  wire    [11:0]  up_hoffset_max_s;
  wire    [11:0]  up_hoffset_min_s;
  wire    [11:0]  up_hoffset_step_s;
  wire    [31:0]  up_startaddr_s;
  wire    [15:0]  up_hsize_s;
  wire    [15:0]  up_hmax_s;
  wire    [15:0]  up_hmin_s;
  wire            es_drp_en_s;
  wire            es_drp_we_s;
  wire    [ 8:0]  es_drp_addr_s;
  wire    [15:0]  es_drp_wdata_s;
  wire    [15:0]  es_drp_rdata_s;
  wire            es_drp_ready_s;
  wire            es_valid_s;
  wire            es_sos_s;
  wire            es_eos_s;
  wire    [31:0]  es_data_s;
  wire            es_state_s;
  wire            mb_ovf_s;
  wire            mb_unf_s;
  wire            mb_state_s;
  wire    [ 1:0]  mb_status_s;
  wire            up_drp_en_s;
  wire            up_drp_we_s;
  wire    [ 8:0]  up_drp_addr_s;
  wire    [15:0]  up_drp_wdata_s;
  wire    [15:0]  up_drp_rdata_s;
  wire            up_drp_ready_s;
  wire            jesd_pre_s;
  wire            gtx_pre_s;
  wire            jesd_rst_s;
  wire            gtx_rst_s;
  wire    [ 3:0]  jesd_rxsync_s;
  wire    [ 3:0]  jesd_rxsof_s;
  wire    [ 3:0]  jesd_rxeof_s;
  wire    [ 3:0]  jesd_rxferr0_s;
  wire    [31:0]  jesd_rxfdata0_s;
  wire    [31:0]  jesd_test_errcnt_s;
  wire    [31:0]  jesd_test_ilacnt_s;
  wire    [31:0]  jesd_test_mfcnt_s;
  wire    [31:0]  jesd_init_data0_s;
  wire    [31:0]  jesd_init_data1_s;
  wire    [31:0]  jesd_init_data2_s;
  wire    [31:0]  jesd_init_data3_s;
  wire    [ 7:0]  jesd_bufcnt_s;
  wire            gtx_rxrstdone_s;
  wire            gtx_rxclk_s;
  wire    [31:0]  gtx_rxdata_s;
  wire    [ 3:0]  gtx_rxcharisk_s;
  wire    [ 3:0]  gtx_rxdisperr_s;
  wire    [ 3:0]  gtx_rxnotintable_s;
  wire            gtx_rxencommaalign_s;
  wire            gtx_rxenchanbond_s;
  wire            gtx_rxchanbondseq_s;
  wire            rx_sysref_enb_s;
  wire            rx_sync_enb_s;
  wire    [ 3:0]  rx_fsm_s;
  wire            rx_align_err_s;

  // debug interface

  assign debug_trigger[5] = rx_sysref;
  assign debug_trigger[4] = rx_sync;

  assign debug_data[107:104] = jesd_rxsof_s;
  assign debug_data[103:100] = jesd_rxeof_s;
  assign debug_data[ 99: 96] = jesd_rxferr0_s;
  assign debug_data[ 95: 64] = jesd_rxfdata0_s;
  assign debug_data[ 63: 63] = rx_sysref;
  assign debug_data[ 62: 62] = rx_sync;
  assign debug_data[ 61: 61] = jesd_rst_s;
  assign debug_data[ 60: 60] = gtx_rst_s;
  assign debug_data[ 59: 56] = jesd_rxsync_s;

  // processor interface

  cf_jesd_regmap #(.NUM_OF_LANES(1)) i_jesd_regmap (
    .up_lanesync_enb (up_lanesync_enb_s),
    .up_scr_enb (up_scr_enb_s),
    .up_sysref_enb (up_sysref_enb_s),
    .up_err_disb (up_err_disb_s),
    .up_frmcnt (up_frmcnt_s),
    .up_bytecnt (up_bytecnt_s),
    .up_bufdelay (up_bufdelay_s),
    .up_test_mode (up_test_mode_s),
    .up_gtx_rstn (up_gtx_rstn_s),
    .up_jesd_rstn (up_jesd_rstn_s),
    .up_es_mode (up_es_mode_s),
    .up_es_start (up_es_start_s),
    .up_prescale (up_prescale_s),
    .up_voffset_step (up_voffset_step_s),
    .up_voffset_max (up_voffset_max_s),
    .up_voffset_min (up_voffset_min_s),
    .up_hoffset_max (up_hoffset_max_s),
    .up_hoffset_min (up_hoffset_min_s),
    .up_hoffset_step (up_hoffset_step_s),
    .up_startaddr (up_startaddr_s),
    .up_hsize (up_hsize_s),
    .up_hmax (up_hmax_s),
    .up_hmin (up_hmin_s),
    .es_drp_en (es_drp_en_s),
    .es_drp_we (es_drp_we_s),
    .es_drp_addr (es_drp_addr_s),
    .es_drp_wdata (es_drp_wdata_s),
    .es_drp_rdata (es_drp_rdata_s),
    .es_drp_ready (es_drp_ready_s),
    .es_state (es_state_s),
    .mb_ovf (mb_ovf_s),
    .mb_unf (mb_unf_s),
    .mb_status (mb_status_s),
    .mb_state (mb_state_s),
    .up_drp_en (up_drp_en_s),
    .up_drp_we (up_drp_we_s),
    .up_drp_addr (up_drp_addr_s),
    .up_drp_wdata (up_drp_wdata_s),
    .up_drp_rdata (up_drp_rdata_s),
    .up_drp_ready (up_drp_ready_s),
    .jesd_status (rx_status),
    .jesd_bufcnt (jesd_bufcnt_s),
    .jesd_init_data0 (jesd_init_data0_s),
    .jesd_init_data1 (jesd_init_data1_s),
    .jesd_init_data2 (jesd_init_data2_s),
    .jesd_init_data3 (jesd_init_data3_s),
    .jesd_test_mfcnt (jesd_test_mfcnt_s),
    .jesd_test_ilacnt (jesd_test_ilacnt_s),
    .jesd_test_errcnt (jesd_test_errcnt_s),
    .up_rstn (up_rstn),
    .up_clk (up_clk),
    .up_sel (up_sel),
    .up_rwn (up_rwn),
    .up_addr (up_addr),
    .up_wdata (up_wdata),
    .up_rdata (up_rdata),
    .up_ack (up_ack));

  // generate sysref and sync controlled by the monitor state machine

  always @(posedge rx_clk) begin
    if (jesd_rst_s == 1'b1) begin
      rx_sysref_enb <= 'd0;
      rx_sysref_enb_d <= 'd0;
      rx_sysref <= 'd0;
      rx_sync_enb <= 'd0;
      rx_sync <= 'd0;
      rx_status <= 'd0;
    end else begin
      rx_sysref_enb <= rx_sysref_enb_s;
      rx_sysref_enb_d <= rx_sysref_enb;
      rx_sysref <= rx_sysref_enb & ~rx_sysref_enb_d;
      rx_sync_enb <= rx_sync_enb_s;
      rx_sync <= (jesd_rxsync_s == 4'b1111) ? rx_sync_enb : 1'b0;
      rx_status <= ((rx_fsm_s == 4'h4) && (rx_align_err_s == 1'h0)) ? 1'b1 : 1'b0;
    end
  end

  // the gtx is held on reset until processor is up.
  // the jesd core is held on reset until after gtx is locked and running

  assign jesd_pre_s = ~(gtx_rxrstdone_s & up_jesd_rstn_s);
  assign gtx_pre_s = ~up_gtx_rstn_s;

  FDPE #(.INIT(1'b1)) i_jesd_rst_reg (
    .CE (1'b1),
    .D (1'b0),
    .PRE (jesd_pre_s),
    .C (rx_clk),
    .Q (jesd_rst_s));

  FDPE #(.INIT(1'b1)) i_gtx_rst_reg (
    .CE (1'b1),
    .D (1'b0),
    .PRE (gtx_pre_s),
    .C (up_clk),
    .Q (gtx_rst_s));

  // this is the receive clock buffer, (the user clock as per xilinx definitions)

  BUFG i_adc_clk_bufg (
    .I (gtx_rxclk_s),
    .O (rx_clk));

  // jesd core (xilinx), documentation and pin definitions are available on-line

  jesd204b_rx1 i_jesd204b_rx1 (
    .clk (rx_clk),
    .rst (jesd_rst_s),
    .rxdata (gtx_rxdata_s),
    .rxcharisk (gtx_rxcharisk_s),
    .rxdisperr (gtx_rxdisperr_s),
    .rxnotintable (gtx_rxnotintable_s),
    .encommaalign (gtx_rxencommaalign_s),
    .enchanbond (gtx_rxenchanbond_s),
    .chanbondseq (gtx_rxchanbondseq_s),
    .sysref_in (rx_sysref),
    .sysref_out (),
    .rx_sync (jesd_rxsync_s),
    .start_of_frame (jesd_rxsof_s),
    .end_of_frame (jesd_rxeof_s),
    .frame_error (jesd_rxferr0_s),
    .rxdataout (jesd_rxfdata0_s),
    .support_lane_sync (up_lanesync_enb_s),
    .scram_enable (up_scr_enb_s),
    .sysref_always (up_sysref_enb_s),
    .disable_error_reporting (up_err_disb_s),
    .octets_per_frame (up_bytecnt_s),
    .frames_per_multiframe (up_frmcnt_s),
    .rx_buffer_delay (up_bufdelay_s),
    .test_modes (up_test_mode_s),
    .test_err_count (jesd_test_errcnt_s),
    .test_ila_count (jesd_test_ilacnt_s),
    .test_mf_count (jesd_test_mfcnt_s),
    .init0 (jesd_init_data0_s),
    .init1 (jesd_init_data1_s),
    .init2 (jesd_init_data2_s),
    .init3 (jesd_init_data3_s),
    .rx_buffer_adjust (jesd_bufcnt_s),
    .rxstatus ());

  // jesd alignment- per lane (all lanes share the same sof/eof)

  cf_jesd_align_2 i_align_2 (
    .rx_clk (rx_clk),
    .rx_sof (jesd_rxsof_s),
    .rx_eof (jesd_rxeof_s),
    .rx_ferr (jesd_rxferr0_s),
    .rx_fdata (jesd_rxfdata0_s),
    .rx_err (rx_align_err_s),
    .rx_data (rx_data0));

  // gtx core (xilinx), this is the key element in this interface (though under-stated
  // by xilinx. The files generated by the coregen are hard to follow through and is
  // a mix of testbench and rtl, so a little hand-editing is done after being
  // generated by coregen, but most of it is the same.

  jesd204b_rx1_gtwizard_v2_1_top # (.EXAMPLE_SIM_GTRESET_SPEEDUP("TRUE")) i_gtx_rx1 (
    .txreset (gtx_rst_s),
    .rxreset (gtx_rst_s),
    .refclk (ref_clk_in),
    .txp_out (),
    .txn_out (),
    .rxp_in (rx_data_p),
    .rxn_in (rx_data_n),
    .tx_reset_done (), 
    .rx_reset_done (gtx_rxrstdone_s), 
    .txoutclk (gtx_rxclk_s),
    .rxoutclk (),
    .txusrclk (rx_clk), 
    .txusrclkrdy (1'b1), 
    .rxusrclk (rx_clk), 
    .rxusrclkrdy (1'b1), 
    .txdata (32'd0),
    .txcharisk (4'd0),
    .rxdata (gtx_rxdata_s),
    .rxcharisk (gtx_rxcharisk_s),
    .rxdisperr (gtx_rxdisperr_s),
    .rxnotintable (gtx_rxnotintable_s),
    .rxencommaalign (gtx_rxencommaalign_s),
    .rxenchanbond (gtx_rxenchanbond_s),
    .rxchanbondseq (gtx_rxchanbondseq_s),
    .drpclk (up_clk),
    .gt0_drpaddr (up_drp_addr_s), 
    .gt0_drpdi (up_drp_wdata_s),   
    .gt0_drpen (up_drp_en_s),    
    .gt0_drpwe (up_drp_we_s),
    .gt0_drpdo (up_drp_rdata_s),
    .gt0_drprdy (up_drp_ready_s));

  // GTX eye scan (per GTX)

  cf_gtx_es_if i_gtx_es_if (
    .up_rstn (up_rstn),
    .up_clk (up_clk),
    .up_start (up_es_start_s),
    .up_mode (up_es_mode_s),
    .up_prescale (up_prescale_s),
    .up_hoffset_min (up_hoffset_min_s),
    .up_hoffset_max (up_hoffset_max_s),
    .up_hoffset_step (up_hoffset_step_s),
    .up_voffset_min (up_voffset_min_s),
    .up_voffset_max (up_voffset_max_s),
    .up_voffset_step (up_voffset_step_s),
    .up_drp_en (es_drp_en_s),
    .up_drp_we (es_drp_we_s),
    .up_drp_addr (es_drp_addr_s),
    .up_drp_wdata (es_drp_wdata_s),
    .up_drp_rdata (es_drp_rdata_s),
    .up_drp_ready (es_drp_ready_s),
    .es_valid (es_valid_s),
    .es_sos (es_sos_s),
    .es_eos (es_eos_s),
    .es_data (es_data_s),
    .es_state (es_state_s),
    .es_dbg_data (es_dbg_data),
    .es_dbg_trigger (es_dbg_trigger));

  // Eye scan write for master

  cf_gtx_es_wr i_gtx_es_wr (
    .up_rstn (up_rstn),
    .up_clk (up_clk),
    .up_startaddr (up_startaddr_s),
    .up_hsize (up_hsize_s),
    .up_hmin (up_hmin_s),
    .up_hmax (up_hmax_s),
    .es_valid (es_valid_s),
    .es_sos (es_sos_s),
    .es_eos (es_eos_s),
    .es_data (es_data_s),
    .mb_req (mb_req),
    .mb_ack (mb_ack),
    .mb_addr (mb_addr),
    .mb_data (mb_data),
    .mb_done (mb_done),
    .mb_error (mb_error),
    .mb_readyn (mb_readyn),
    .mb_ovf (mb_ovf_s),
    .mb_unf (mb_unf_s),
    .mb_state (mb_state_s),
    .mb_status (mb_status_s),
    .mb_dbg_data (mb_dbg_data),
    .mb_dbg_trigger (mb_dbg_trigger));

  // JESD monitor

  cf_jesd_mon i_mon (
    .rx_rst (jesd_rst_s),
    .rx_clk (rx_clk),
    .rx_data (gtx_rxdata_s),
    .rx_charisk (gtx_rxcharisk_s),
    .rx_disperr (gtx_rxdisperr_s),
    .rx_notintable (gtx_rxnotintable_s),
    .rx_sysref (rx_sysref),
    .rx_sync (rx_sync),
    .rx_sysref_enb (rx_sysref_enb_s),
    .rx_sync_enb (rx_sync_enb_s),
    .rx_fsm (rx_fsm_s),
    .mon_trigger (debug_trigger[3:0]),
    .mon_data (debug_data[55:0]));

endmodule

// ***************************************************************************
// ***************************************************************************
